""" Workspace permissions """

from aurori.workspaces import Permission

# Define your Permissions here


class ViewLogs(Permission):
    """View logs permission"""

    description = "Allow to see the logs"
